/*
 *
 * X DLX Simulator
 *
 * The DLX is a 32-bit RISC machine based on the MIPS 2000 architecture.
 * The architecture includes 32 32-bit general purpose registers and a 
 * program counter.
 *
 * Floating point registers and an interrupt register are not implemented.
 *
 * The address space is 4 Gbytes, big-endian byte-addressable using 32-bit
 * addresses. All instructions must be properly aligned.
 *
 *
 * Cameron Humphries
 * Department of Computer Science
 * University of Adelaide
 * 1992
 *
 */
 
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/SimpleMenu.h>

#define NO_SELECTION 0
#define CODE_SELECTED 1
#define DATA_SELECTED 2

#define SHOW_BYTES 1
#define SHOW_HALFWORDS 2
#define SHOW_WORDS 3

extern Widget data_list;
extern Widget code_list;

extern int selection_type;
extern int mem_loc_select;
extern int data_type;
extern void Write_Comm_Channel_Str();
extern void Write_Comm_Channel_Int();
extern void Write_Status_Message();

static void set_rb(),clr_rb();
static void set_wb(),clr_wb();
static void set_fb(),clr_fb();
static void clr_all();

/*
 * Create menu button for popup breakpoint menu.
 */
void Create_BrkPt_Menu(Widget form)
{
  Widget w,breakpoint_menu_popup;
  static Arg arg_list[] = {
    {XtNlabel, (XtArgVal)"BREAKPOINT"},
    {XtNfromHoriz, (XtArgVal)NULL},
    {XtNmenuName, (XtArgVal)"Breakpoint_Menu_Popup"},
  };
  static Arg menu_entry_arg[] = {
    {XtNlabel, (XtArgVal)NULL},
    {XtNcallback, (XtArgVal)NULL},
  };
  static XtCallbackRec set_rb_cbks[] = {
    {set_rb, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec clr_rb_cbks[] = {
    {clr_rb, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec set_wb_cbks[] = {
    {set_wb, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec clr_wb_cbks[] = {
    {clr_wb, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec set_fb_cbks[] = {
    {set_fb, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec clr_fb_cbks[] = {
    {clr_fb, NULL},
    {NULL,NULL}
  };
  static XtCallbackRec clr_all_cbks[] = {
    {clr_all, NULL},
    {NULL,NULL}
  };
    
  /* Create Popup Menu */
  breakpoint_menu_popup = XtCreatePopupShell("Breakpoint_Menu_Popup",
					     simpleMenuWidgetClass,form,
					     NULL,0);
					     
  /* Create Menu entries */
  menu_entry_arg[0].value = (XtArgVal)"Set Fetch Breakpoint";
  menu_entry_arg[1].value = (XtArgVal)set_fb_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  menu_entry_arg[0].value = (XtArgVal)"Clear Fetch Breakpoint"; 
  menu_entry_arg[1].value = (XtArgVal)clr_fb_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  w = XtCreateManagedWidget("Line_1",smeLineObjectClass,breakpoint_menu_popup,
			    NULL,0);

  menu_entry_arg[0].value = (XtArgVal)"Set Read Breakpoint";
  menu_entry_arg[1].value = (XtArgVal)set_rb_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  menu_entry_arg[0].value = (XtArgVal)"Clear Read Breakpoint";
  menu_entry_arg[1].value = (XtArgVal)clr_rb_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  w = XtCreateManagedWidget("Line_2",smeLineObjectClass,breakpoint_menu_popup,
			    NULL,0);

  menu_entry_arg[0].value = (XtArgVal)"Set Write Breakpoint";
  menu_entry_arg[1].value = (XtArgVal)set_wb_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  menu_entry_arg[0].value = (XtArgVal)"Clear Write Breakpoint";
  menu_entry_arg[1].value = (XtArgVal)clr_wb_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  w = XtCreateManagedWidget("Line_3",smeLineObjectClass,breakpoint_menu_popup,
			    NULL,0);

  menu_entry_arg[0].value = (XtArgVal)"Clear All Breakpoint(s)";
  menu_entry_arg[1].value = (XtArgVal)clr_all_cbks;
  w = XtCreateManagedWidget("BM_SFB",smeBSBObjectClass,breakpoint_menu_popup,
			    menu_entry_arg,XtNumber(menu_entry_arg));
			    
  /* Create Menu button */
  arg_list[1].value = (XtArgVal)XtNameToWidget(form,"Execute_Menu");
  w = XtCreateManagedWidget("BrkPt_Menu",menuButtonWidgetClass,form,
			    arg_list,XtNumber(arg_list));
  
  return;
}

static void set_rb(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  int mem_loc;
  
  if ((selection_type == NO_SELECTION) || (selection_type == CODE_SELECTED))
    {
/*      XBell(XtDisplay(w),50); */
    }
  else
    {
      switch (data_type)
	{
	case SHOW_BYTES:
	  mem_loc = mem_loc_select;
	  break;
	case SHOW_HALFWORDS:
	  mem_loc = mem_loc_select*2;
	  break;
	case SHOW_WORDS:
	  mem_loc = mem_loc_select*4;
	  break;
	}	  

      sprintf(str,"Set read brkpt on location %d",mem_loc);
      (void)Write_Status_Message(str);
      Write_Comm_Channel_Str("br");
      Write_Comm_Channel_Int(mem_loc);
      XawMemListAddBrkPt(data_list,mem_loc_select);
    }

  return;
}

static void clr_rb(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  int mem_loc;
  
  if ((selection_type == NO_SELECTION) || (selection_type == CODE_SELECTED))
    {
/*      XBell(XtDisplay(w),50); */
    }
  else
    {
      switch (data_type)
	{
	case SHOW_BYTES:
	  mem_loc = mem_loc_select;
	  break;
	case SHOW_HALFWORDS:
	  mem_loc = mem_loc_select*2;
	  break;
	case SHOW_WORDS:
	  mem_loc = mem_loc_select*4;
	  break;
	}	  

      sprintf(str,"Clear read brkpt on location %d",mem_loc);
      (void)Write_Status_Message(str);
      Write_Comm_Channel_Str("dr");
      Write_Comm_Channel_Int(mem_loc);
      XawMemListDelBrkPt(data_list,mem_loc_select);
    }


  return;
}

static void set_wb(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  int mem_loc;
  
  if ((selection_type == NO_SELECTION) || (selection_type == CODE_SELECTED))
    {
/*      XBell(XtDisplay(w),50); */
    }
  else
    {
      switch (data_type)
	{
	case SHOW_BYTES:
	  mem_loc = mem_loc_select;
	  break;
	case SHOW_HALFWORDS:
	  mem_loc = mem_loc_select*2;
	  break;
	case SHOW_WORDS:
	  mem_loc = mem_loc_select*4;
	  break;
	}	  

      sprintf(str,"Set write brkpt on location %d",mem_loc);
      (void)Write_Status_Message(str);
      Write_Comm_Channel_Str("bw");
      Write_Comm_Channel_Int(mem_loc);
      XawMemListAddBrkPt(data_list,mem_loc_select);
    }


  return;
}

static void clr_wb(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  int mem_loc;
    
  if ((selection_type == NO_SELECTION) || (selection_type == CODE_SELECTED))
    {
/*      XBell(XtDisplay(w),50); */
    }
  else
    {
      switch (data_type)
	{
	case SHOW_BYTES:
	  mem_loc = mem_loc_select;
	  break;
	case SHOW_HALFWORDS:
	  mem_loc = mem_loc_select*2;
	  break;
	case SHOW_WORDS:
	  mem_loc = mem_loc_select*4;
	  break;
	}	  

      sprintf(str,"Clear write brkpt on location %d",mem_loc);
      (void)Write_Status_Message(str);
      Write_Comm_Channel_Str("dw");
      Write_Comm_Channel_Int(mem_loc);
      XawMemListDelBrkPt(data_list,mem_loc_select);
    }


  return;
}

static void set_fb(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  int mem_loc;
  
  if ((selection_type == NO_SELECTION))
    {
/*      XBell(XtDisplay(w),50); */
    }
  else
    {
      if (selection_type == DATA_SELECTED)
	{
	  switch (data_type)
	    {
	    case SHOW_BYTES:
	      mem_loc = mem_loc_select;
	      break;
	    case SHOW_HALFWORDS:
	      mem_loc = mem_loc_select*2;
	      break;
	    case SHOW_WORDS:
	      mem_loc = mem_loc_select*4;
	      break;
	      XawMemListAddBrkPt(data_list,mem_loc_select);
	    }	  
	}
      else
	{
	  mem_loc = mem_loc_select*4;
	  XawMemListAddBrkPt(code_list,mem_loc_select);
	}

      sprintf(str,"Set fetch brkpt on location %d",mem_loc);
      (void)Write_Status_Message(str);
      Write_Comm_Channel_Str("bf");
      Write_Comm_Channel_Int(mem_loc);
    }


  return;
}

static void clr_fb(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  int mem_loc;
  
  if ((selection_type == NO_SELECTION))
    {
/*      XBell(XtDisplay(w),50); */
    }
  else
    {
      if (selection_type == DATA_SELECTED)
	{
	  switch (data_type)
	    {
	    case SHOW_BYTES:
	      mem_loc = mem_loc_select;
	      break;
	    case SHOW_HALFWORDS:
	      mem_loc = mem_loc_select*2;
	      break;
	    case SHOW_WORDS:
	      mem_loc = mem_loc_select*4;
	      break;
	    }	  
	  XawMemListDelBrkPt(data_list,mem_loc_select);
	}
      else
	{
	  mem_loc = mem_loc_select*4;
	  XawMemListDelBrkPt(code_list,mem_loc_select);
	}

      sprintf(str,"Clear fetch brkpt on location %d",mem_loc);
      (void)Write_Status_Message(str);
      Write_Comm_Channel_Str("df");
      Write_Comm_Channel_Int(mem_loc);
    }


  return;
}

static void clr_all(Widget w,XtPointer client_data, XtPointer call_data)
{
  char str[200];
  
  sprintf(str,"Clearing all brkpt");
  (void)Write_Status_Message(str);
  Write_Comm_Channel_Str("c");
  XawMemListReset(data_list);
  XawMemListReset(code_list);
  
  return;
}

